/////////////////////////////////////////////////////////////////////////////////
//
//  This file is part of the TLIB computer vision library.
//  Copyright (C) 2003-2008 Sebastien Grange
//  Copyright (C) 2003-2007 VRAI Group, EPFL
//  All rights reserved.
// 
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License("GPL") version 2
//  as published by the Free Software Foundation.
// 
//  \description
//  This example extracts and highlights blobs (i.e. contiguous groups of pixels)
//  that belong to saturated regions of the image. You can modifiy the saturation
//  threshold by changing the '254' argument on line 65.
//
//  <http://www.tuyphon.com/tlib>
//
/////////////////////////////////////////////////////////////////////////////////



#include "tlDigitizer.h"
#include "tlDisplay.h"
#include "tlVision.h"



int 
main(int  argc, 
     char **argv)
{
  int           i;
  bool          done        = false;
  tlDisplay     *window     = new tlDisplay    ();
  tlImage       *image      = new tlImage      ();
  tlImage       *grayImage  = new tlImage      ();
  tlBlobs       *blobs      = new tlBlobs      ();
  TL_DIGITIZER  *digitizer  = new TL_DIGITIZER ();


  // initialize TLIB
  tl_init_lib (argc, argv);
  tl_set_verbose (1);

  // initialize objects
  window->show ();
  if (digitizer->open(0) != TL_NO_ERROR) {
    TL_ERROR_MSG ("error opening digitizer\n");
    return TL_ERROR;
  }

  // the faster the better !
  digitizer->setFrameRate (60);

  // main loop
  while (!done) 
  {
    // grab image
    digitizer->grab (image);

    // find saturated areas
    image->toGray (grayImage);
    grayImage->normalize ();
    grayImage->toBin (254);
    grayImage->dilate (10);

    // extract blobs of saturated areas bigger than 50 pixels
    blobs->extract (grayImage, 50);

    // draw blobs
    for (i=0; i<blobs->getCount (); i++) {
      image->drawMask (blobs->getBlob (i), 128+i*255/blobs->getCount (), 255-i*255/blobs->getCount (), i*255/blobs->getCount ());
    }

    // display image
    window->display (image);

    // event management
    if (tlDisplay::queryEvent ()) {
      switch (tlDisplay::event ()) {
      case TL_EVENT_EXIT:
        done = true;
        break;
      case TL_EVENT_KEYUP:
        switch (tlDisplay::getKeyUp ()) {
        case 'q':
        case 'Q':
          done = true;
          break;
        }
      }
    }
  }


  // cleanup
  delete image;
  delete blobs;
  delete digitizer;
  delete window;
  printf("done.\n");


  return 0;
}

