/////////////////////////////////////////////////////////////////////////////////
//
//  This file is part of the TLIB computer vision library.
//  Copyright (C) 2003-2008 Sebastien Grange
//  Copyright (C) 2003-2007 VRAI Group, EPFL
//  All rights reserved.
// 
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License("GPL") version 2
//  as published by the Free Software Foundation.
// 
//  <http://www.tuyphon.com/tlib>
//
/////////////////////////////////////////////////////////////////////////////////


#include <stdio.h>
#include <stdlib.h>

#include "tlDigitizer.h"
#include "tlDisplay.h"
#include "tlVision.h"

// globals
bool            done        = false;
tlDisplay       *window     = NULL;
TL_DIGITIZER    *digitizer;



// Initialization routine

int 
init ()
{
  // library init
  tl_init_lib ();

  // open windows
  window = new tlDisplay ("live");

  // open digitizer
  digitizer = new TL_DIGITIZER();
  if (digitizer->open() != TL_NO_ERROR) {
    TL_ERROR_MSG ("error opening digitizer\n");
    return TL_ERROR;
  }

  // show windows
  window->show ();


  return TL_NO_ERROR;
}



// Clean up routine

int 
cleanup (void)
{
  delete digitizer;
  delete window;


  return TL_NO_ERROR;
}



// Image processing loop, run in its own high-priority thread.

void 
process (void)
{
  // locals
  tlImage *image   = new tlImage ();


  while (!done) 
  {
    // event management
    if (tlDisplay::queryEvent () != TL_NO_EVENT) {
      switch (tlDisplay::event()) {
      case TL_NO_EVENT:
        break;
      case TL_EVENT_KEYUP:
        switch (tlDisplay::getKeyUp ()) {
      case 'Q':
        done = true;
        break;
        }
        break;
      }
    }

    // grab image
    if ((digitizer->grab (image)) != TL_NO_ERROR) {
      TL_ERROR_MSG ("error - grab failed");
      break;
    }

    // display image
    if ((window->display (image)) != TL_NO_ERROR) {
      if (!done) TL_ERROR_MSG ("error - display failed");
      break;
    }
  }


  // termination
  delete image;
  tlThreadExit (42);
}



//  Event manager

void 
event (void)
{  
  while (!done) {
    if (tlDisplay::queryEvent() == TL_EVENT) {
      switch (tlDisplay::event ()) {
      case TL_EVENT_EXIT:
      case TL_EVENT_KEYUP:
        switch (tlDisplay::getKeyUp ()) {
        case 'Q':
        case 'q':
          done = true;
          break;
        }
        break;
      }
    }
  }
}



// Application entry point

int 
main(int  argc, 
     char **argv)
{
  tlThreadHandle  thread;
  int             res;


  // initialization
  if (init () != TL_NO_ERROR) exit(1);

  // main routine
  tlThreadCreate (process, &thread);
  event ();

  // safe exit
  if (tlThreadJoin (&thread, &res, 3000) == TL_ERROR_TIMEOUT) {
    tlThreadKill (&thread);
    res = -1;
  }

  // cleanly exit
  cleanup ();
  printf("done.\n");


  return (res);
}
