/////////////////////////////////////////////////////////////////////////////////
//
//  This file is part of the TLIB computer vision library.
//  Copyright (C) 2003-2008 Sebastien Grange
//  Copyright (C) 2003-2007 VRAI Group, EPFL
//  All rights reserved.
// 
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License("GPL") version 2
//  as published by the Free Software Foundation.
// 
//  <http://www.tuyphon.com/tlib>
//
/////////////////////////////////////////////////////////////////////////////////



#include <stdlib.h>

#include "tlDigitizer.h"
#include "tlDisplay.h"
#include "tlTimer.h"
#include "tlVision.h"

// globals
tlDisplay     *display    = NULL;
TL_DIGITIZER  *digitizer;



// Initialization

int init ()
{
  // library init
  tl_init_lib ();

  // verbosity mode
  tl_set_verbose (0);

  // open window
  display = new tlDisplay ("SimpleApp");

  // open digitizer
  digitizer = new TL_DIGITIZER();

  if (digitizer->open() != TL_NO_ERROR) {
    TL_ERROR_MSG ("error opening digitizer\n");
    return TL_ERROR;
  }

#ifdef LINUX
  if (digitizer->setFrameRate (30) != TL_NO_ERROR) {
    TL_ERROR_MSG ("error - digitizer failed to set frame rate");
    return TL_ERROR;
  }
#endif

  // show windows
  display->show ();
  

  return TL_NO_ERROR;
}



// Clean up

int cleanup ()
{
  delete digitizer;
  delete display;


  return TL_NO_ERROR;
}



// Application entry point

int
main (int  argc, 
      char **argv)
{
  int     done    = 0;
  int     frame   = 0;
  tlImage *image  = new tlImage ();


  // initialization
  if (init () != TL_NO_ERROR) {
    exit (1);
  }

  // main loop
  while (!done) {
    // image processing
    digitizer->grab (image);
    display->display (image);

    // event processing
    if (tlDisplay::queryEvent() == TL_EVENT) {
      switch (tlDisplay::event ()) {
      case TL_EVENT_KEYDOWN:
        if (tlDisplay::getKeyDown() == 'q') {
          done = 1;
        }
	      break;
      case TL_EVENT_EXIT:
        done = 1;
	      break;
      }
    }

    // statistics 
    if (++frame == 1) {
      if (tl_timer_init() != TL_NO_ERROR) {
        exit (1);
      }
    }
    if (!(frame % 30)) {
      printf ("%.1f frames/sec\n", (float) frame / ((float) tl_timer_get_time() / 1000.0));
    }
  }

  // cleanly exit
  delete image;

  cleanup ();
  printf("done.\n");


  return 0;
}
