/////////////////////////////////////////////////////////////////////////////////
//
//  This file is part of the TLIB computer vision library.
//  Copyright (C) 2003-2008 Sebastien Grange
//  Copyright (C) 2003-2007 VRAI Group, EPFL
//  All rights reserved.
// 
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License("GPL") version 2
//  as published by the Free Software Foundation.
// 
//  <http://www.tuyphon.com/tlib>
//
/////////////////////////////////////////////////////////////////////////////////


#ifndef __TLDIGITIZERDX_H__
#define __TLDIGITIZERDX_H__



#include "tlVision.h"
#include "tlDigitizer.h"

// DirectX headers
#ifndef __TLDIRECTXH__
#define __TLDIRECTXH__
#include <atlbase.h>
#include <qedit.h>
#include <dshow.h>
#ifdef _TLIB_BUILD_
#include <streams.h>
#endif
#endif



class SampleGrabberCallback : public ISampleGrabberCB
{
private:
  bool *ready;

public:
  SampleGrabberCallback (bool *imageReady) { ready = imageReady; };
  virtual ~SampleGrabberCallback () {return; };
  STDMETHODIMP_(ULONG) AddRef  () { return 1; }
  STDMETHODIMP_(ULONG) Release () { return 2; }

  STDMETHODIMP QueryInterface(REFIID riid, void **ppvObject) {
    if (NULL == ppvObject) return E_POINTER;
    if (riid == __uuidof(IUnknown)) {
      *ppvObject = static_cast<IUnknown*>(this);
      return S_OK;
    }
    if (riid == __uuidof(ISampleGrabberCB)) {
      *ppvObject = static_cast<ISampleGrabberCB*>(this);
      return S_OK;
    }
    return E_NOTIMPL;
  }

  STDMETHODIMP SampleCB(double Time, IMediaSample *pSample) {
    return E_NOTIMPL;
  }

  STDMETHODIMP BufferCB(double Time, BYTE *pBuffer, long BufferLen) {
    *ready = true;
    return S_OK;
  }
};



// interface
class tlDigitizerDX : public tlDigitizer
{
private:
  bool                    imageReady;
  SampleGrabberCallback   *cb;

protected:
  ISampleGrabber          *pGrabber;
  IGraphBuilder           *pGraph;
  ICaptureGraphBuilder2   *pCapture;
  BYTE                    *buffer;
  IAMStreamConfig         *pVConfig;
  IMediaEvent             *pEvent;
  int                     index;
  int                     fps;

public:
  tlDigitizerDX  ();
  tlDigitizerDX  (int width, int height);
  virtual ~tlDigitizerDX ();

  virtual int open            (int index = 0);
  virtual int open            (int width, int height, int index = 0);
  virtual int close           ();

  virtual int setSize         (int width, int height);

  virtual int grab            (tlImage *image);
  virtual int grab            (tlImage *image, tlRect *rect);
  virtual int grabQueued      (tlImage *image);
  virtual int grabQueued      (tlImage *image, tlRect *rect);

  int setFrameRate (int fps);
};


#endif

