/////////////////////////////////////////////////////////////////////////////////
//
//  This file is part of the TLIB computer vision library.
//  Copyright (C) 2003-2008 Sebastien Grange
//  Copyright (C) 2003-2007 VRAI Group, EPFL
//  All rights reserved.
// 
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License("GPL") version 2
//  as published by the Free Software Foundation.
// 
//  <http://www.tuyphon.com/tlib>
//
/////////////////////////////////////////////////////////////////////////////////


#ifndef __TLFILE_H__
#define __TLFILE_H__

#include "tlVision.h"

#ifdef LINUX
#include <sys/types.h>

typedef u_int8_t BYTE, *LPBYTE;	/* unsigned char */
typedef u_int16_t WORD;		/* unsigned short */
typedef u_int32_t DWORD;	/* unsigned long */
typedef int32_t   LONG;		/* long */

/* this pragma is intended to tell GCC to byte align (pad) the following
 * structures
 */
#pragma pack(1)

typedef struct tagBITMAPFILEHEADER {
  WORD    bfType;		/* byte 0-1 */
  DWORD   bfSize;		/* 2-5 */
  WORD    bfReserved1;		/* 6-7 */
  WORD    bfReserved2;		/* 8-9 */
  DWORD   bfOffBits;		/* 10-13 */
} BITMAPFILEHEADER;

typedef struct tagBITMAPINFOHEADER {
   DWORD  biSize;
   LONG   biWidth;
   LONG   biHeight;
   WORD   biPlanes;
   WORD   biBitCount;
   DWORD  biCompression;
   DWORD  biSizeImage;
   LONG   biXPelsPerMeter;
   LONG   biYPelsPerMeter;
   DWORD  biClrUsed;
   DWORD  biClrImportant;
} BITMAPINFOHEADER;


typedef struct tagRGBQUAD {
  BYTE    rgbBlue;
  BYTE    rgbGreen;
  BYTE    rgbRed;
  BYTE    rgbReserved;
} RGBQUAD;

/* resume normal packing... */
#pragma pack()

#endif

int tl_write_bmp (tlPixel *src, int width, int height, tl_format format, char *filename);
int tl_write_gif (tlPixel *src, int width, int height, char *filename);
int tl_write_jpg (tlPixel *src, int width, int height, int pixel_width, int quality, char *filename);
int tl_write_ppm (tlPixel *src, int width, int height, char *filename);
int tl_write_tlb (tlImage *image, char *filename);

tlPixel *tl_read_bmp (char *filename, int *width, int *height, tl_format *format);
tlPixel *tl_read_gif (char *filename, int *width, int *height, tl_format *format);
tlPixel *tl_read_jpg (char *filename, int *width, int *height, tl_format *format);
tlPixel *tl_read_ppm (char *filename, int *width, int *height, tl_format *format);
void    *tl_read_tlb (char *filename, int *width, int *height, int *pixel_width, tl_format *format);

#endif
